@@ -537,7 +537,7 @@ class GLTF(object):
             if binary:
                 bytesio.write(data)
             else:
-                json.dump(data, bytesio, sort_keys=True, indent=4, separators=(',', ': '))
+                json.dump(data, bytesio, sort_keys=True, allow_nan=False, indent=4, separators=(',', ': '))
             return bytesio
 
         path = os.path.join(folder, filename)
@@ -546,7 +546,7 @@ class GLTF(object):
                 f.write(data)
         else:
             with open(path, 'w') as f:
-                json.dump(data, f, sort_keys=True, indent=4, separators=(',', ': '))
+                json.dump(data, f, sort_keys=True, allow_nan=False, indent=4, separators=(',', ': '))
         return path
 
     def repair(self):
