@@ -14,16 +14,11 @@ logger = logging.getLogger(__name__)
 
 
 class Primitive(BaseGLTFStructure):
-    _positions = None
-    _normals = None
-    _indices = None
-    _tangents = None
+    attributes = None
     mode = None
     material = None
-    texcoords = None
-    colors = None
-    joints = None
-    weights = None
+    targets = None
+    _indices = None
 
     class Mode(Enum):
         POINTS = 0
@@ -34,9 +29,9 @@ class Primitive(BaseGLTFStructure):
         TRIANGLE_STRIP = 5
         TRIANGLE_FAN = 6
 
-    def __init__(self, prim=None, gltf=None, name=None, joints=None, weights=None,
-                 positions=None, normals=None, indices=None, texcoords=None, tangents=None,
-                 material=None, image=None, texture=None, mode=None, colors=None):
+    def __init__(self, prim=None, gltf=None, name=None, indices=None,
+                 material=None, image=None, texture=None, mode=None,
+                 attributes=None, targets=None, **kwargs):
         self.name = name
         if prim:
             if 'name' in prim:
@@ -49,106 +44,44 @@ class Primitive(BaseGLTFStructure):
                 self.mode = self.Mode(prim.get('mode'))
             attributes = prim.get('attributes')
             if attributes:
-                if 'POSITION' in attributes:
-                    self.positions = gltf.accessors[attributes.get('POSITION')]
-                if 'NORMAL' in attributes:
-                    self.normals = gltf.accessors[attributes.get('NORMAL')]
-                if 'TANGENT' in attributes:
-                    self.tangents = gltf.accessors[attributes.get('TANGENT')]
-                self.texcoords = []
-                if 'TEXCOORD_0' in attributes:
-                    i = 0
-                    while True:
-                        texcoords = attributes.get('TEXCOORD_' + str(i))
-                        if texcoords is None:
-                            break
-                        self.texcoords.append(gltf.accessors[texcoords])
-                        i += 1
-                self.colors = []
-                if 'COLOR_0' in attributes:
-                    i = 0
-                    while True:
-                        colors = attributes.get('COLOR_' + str(i))
-                        if colors is None:
-                            break
-                        self.colors.append(gltf.accessors[colors])
-                        i += 1
-                self.joints = []
-                if 'JOINTS_0' in attributes:
-                    i = 0
-                    while True:
-                        joints = attributes.get('JOINTS_' + str(i))
-                        if joints is None:
-                            break
-                        self.joints.append(gltf.accessors[joints])
-                        i += 1
-                self.weights = []
-                if 'WEIGHTS_0' in attributes:
-                    i = 0
-                    while True:
-                        weights = attributes.get('WEIGHTS_' + str(i))
-                        if weights is None:
-                            break
-                        self.weights.append(gltf.accessors[weights])
-                        i += 1
+                self.attributes = Attributes(attributes, gltf)
+            targets = prim.get('targets')
+            if targets:
+                self.targets = [Attributes(t, gltf) for t in targets]
         else:
-            self.positions = positions
-            self.normals = normals
-            self.indices = indices
-            self.tangents = tangents
             self.material = material
-            self.texcoords = texcoords if texcoords is not None else []
-            self.colors = colors if colors is not None else []
-            self.joints = joints if joints is not None else []
-            self.weights = weights if weights is not None else []
             self.mode = mode and self.Mode[mode]
             if image:
                 self.material = Material(image=image)
             if texture:
                 self.material = Material(color_texture=texture)
+            if attributes:
+                self.attributes = attributes
+            elif kwargs:
+                self.attributes = Attributes(gltf=gltf, **kwargs)
+            else:
+                self.attributes = Attributes()
+            if targets:
+                self.targets = targets
+            if indices:
+                self.indices = indices
 
     def __repr__(self):
         return self.name or super(Primitive, self).__repr__()
 
     def __contains__(self, item):
-        return (self._positions == item or
-                self._normals == item or
-                self._indices == item or
-                self._tangents == item or
-                item in self.texcoords or
-                item in self.colors or
-                item in self.joints or
-                item in self.weights)
+        return (self._indices == item
+                or item in self.attributes
+                or (self.targets and any(item in t for t in self.targets)))
 
     def sort_joints(self):
-        if len(self.joints) > 1:
-            joint_set_count = len(self.joints)
-            joint_data = np.hstack(tuple(j.data for j in self.joints))
-            weight_data = np.hstack(tuple(w.data for w in self.weights))
-            sorted_weights = []
-            sorted_joints = []
-            for w, j in zip(weight_data, joint_data):
-                pairs = sorted(zip(w, j), reverse=True)
-                new_weights, new_joints = list(zip(*pairs))
-                sorted_weights.append(new_weights)
-                sorted_joints.append(new_joints)
-            sorted_weights = np.array(sorted_weights, 'float32')
-            sorted_joints = np.array(sorted_joints, 'uint16')
-            self.weights = []
-            self.joints = []
-            for i in range(joint_set_count):
-                self.weights.append(
-                    Vec4Array(sorted_weights[:, i * 4:(i + 1) * 4].astype('float32'))
-                )
-                self.joints.append(
-                    Vec4Array(sorted_joints[:, i * 4:(i + 1) * 4].astype('uint16'))
-                )
+        self.attributes.sort_joints()
 
     def split_transparency(self, alpha_mode=Material.AlphaMode.BLEND):
         if self.mode and self.mode != self.Mode.TRIANGLES:
             raise NotImplementedError('Prim splitting only implemented for triangles')
 
-        if not self.material or not self.texcoords or (
+        if not self.material or not self.attributes.texcoords or (
                 self.material.alpha_mode == Material.AlphaMode.OPAQUE):
             raise ValueError('Can\'t split prim: no material or texcoords, or material '
                              'is opaque, or material has no color_texture or diffuse_texture.')
@@ -172,9 +105,9 @@ class Primitive(BaseGLTFStructure):
             return
 
         if self.material.color_uv is not None:
-            texcoords = self.texcoords[self.material.color_uv]
+            texcoords = self.attributes.texcoords[self.material.color_uv]
         else:
-            texcoords = self.texcoords[0]
+            texcoords = self.attributes.texcoords[0]
 
         if self.indices:
             indices_iter = (self.indices.data[pos:pos + 3]
@@ -210,33 +143,27 @@ class Primitive(BaseGLTFStructure):
         self.material.alpha_mode = Material.AlphaMode.OPAQUE
         self.indices = o_indices
 
-        return Primitive(positions=self.positions, normals=self.normals,
-                         texcoords=self.texcoords, tangents=self.tangents,
-                         joints=self.joints, weights=self.weights, colors=self.colors,
-                         indices=t_indices, material=t_material)
+
+        attributes = Attributes(positions=self.attributes.positions, normals=self.attributes.normals,
+                                texcoords=self.attributes.texcoords, tangents=self.attributes.tangents,
+                                joints=self.attributes.joints, weights=self.attributes.weights, colors=self.attributes.colors)
+        return Primitive(attributes=attributes, indices=t_indices, material=t_material)
 
     def render(self, gltf):
-        primitive = {'attributes': {}}
+        primitive = {}
         if self.mode:
             primitive['mode'] = self.mode.value
         if self.material:
             primitive['material'] = gltf.index('materials', self.material)
         if self.indices and self.indices.count:
             primitive['indices'] = gltf.index('accessors', self.indices)
-        if self.positions and self.positions.count:
-            primitive['attributes']['POSITION'] = gltf.index('accessors', self.positions)
-        if self.normals and self.normals.count:
-            primitive['attributes']['NORMAL'] = gltf.index('accessors', self.normals)
-        if self.tangents and self.tangents.count:
-            primitive['attributes']['TANGENT'] = gltf.index('accessors', self.tangents)
-        for i, texcoords in enumerate(self.texcoords):
-            primitive['attributes']['TEXCOORD_' + str(i)] = gltf.index('accessors', texcoords)
-        for i, colors in enumerate(self.colors):
-            primitive['attributes']['COLOR_' + str(i)] = gltf.index('accessors', colors)
-        for i, joints in enumerate(self.joints):
-            primitive['attributes']['JOINTS_' + str(i)] = gltf.index('accessors', joints)
-        for i, weights in enumerate(self.weights):
-            primitive['attributes']['WEIGHTS_' + str(i)] = gltf.index('accessors', weights)
+        if self.attributes:
+            primitive['attributes'] = self.attributes.render(gltf)
+        if self.targets:
+            targets = []
+            for target in self.targets:
+                targets.append(target.render(gltf))
+            primitive['targets'] = targets
         return primitive
 
     def repair(self):
@@ -274,6 +201,188 @@ class Primitive(BaseGLTFStructure):
                         norm = dir / mag
                     self.normals.data[index] = norm
 
+    @property
+    def positions(self):
+        return self.attributes.positions
+
+    @positions.setter
+    def positions(self, positions):
+        self.attributes.positions = positions
+
+    @property
+    def normals(self):
+        return self.attributes.normals
+
+    @normals.setter
+    def normals(self, normals):
+        self.attributes.normals = normals
+
+    @property
+    def tangents(self):
+        return self.attributes.tangents
+
+    @tangents.setter
+    def tangents(self, tangents):
+        self.attributes.tangents = tangents
+
+    @property
+    def texcoords(self):
+        return self.attributes.texcoords
+
+    @texcoords.setter
+    def texcoords(self, val):
+        self.attributes.texcoords = val
+
+    @property
+    def colors(self):
+        return self.attributes.colors
+
+    @colors.setter
+    def colors(self, val):
+        self.attributes.colors = val
+
+    @property
+    def joints(self):
+        return self.attributes.joints
+
+    @joints.setter
+    def joints(self, val):
+        self.attributes.joints = val
+
+    @property
+    def weights(self):
+        return self.attributes.weights
+
+    @weights.setter
+    def weights(self, val):
+        self.attributes.weights = val
+
+    @property
+    def indices(self):
+        return self._indices
+
+    @indices.setter
+    def indices(self, indices):
+        if indices is None:
+            self._indices = None
+            return
+        self._indices = indices if isinstance(indices, ScalarArray) else ScalarArray(indices)
+
+
+class Attributes:
+    _positions = None
+    _normals = None
+    _tangents = None
+    texcoords = None
+    colors = None
+    joints = None
+    weights = None
+
+    def __init__(self, attrs=None, gltf=None, positions=None, normals=None, tangents=None, 
+                 texcoords=None, colors=None, joints=None, weights=None):
+        if attrs:
+            if 'POSITION' in attrs:
+                self.positions = gltf.accessors[attrs.get('POSITION')]
+            if 'NORMAL' in attrs:
+                self.normals = gltf.accessors[attrs.get('NORMAL')]
+            if 'TANGENT' in attrs:
+                self.tangents = gltf.accessors[attrs.get('TANGENT')]
+            self.texcoords = []
+            if 'TEXCOORD_0' in attrs:
+                i = 0
+                while True:
+                    texcoords = attrs.get('TEXCOORD_' + str(i))
+                    if texcoords is None:
+                        break
+                    self.texcoords.append(gltf.accessors[texcoords])
+                    i += 1
+            self.colors = []
+            if 'COLOR_0' in attrs:
+                i = 0
+                while True:
+                    colors = attrs.get('COLOR_' + str(i))
+                    if colors is None:
+                        break
+                    self.colors.append(gltf.accessors[colors])
+                    i += 1
+            self.joints = []
+            if 'JOINTS_0' in attrs:
+                i = 0
+                while True:
+                    joints = attrs.get('JOINTS_' + str(i))
+                    if joints is None:
+                        break
+                    self.joints.append(gltf.accessors[joints])
+                    i += 1
+            self.weights = []
+            if 'WEIGHTS_0' in attrs:
+                i = 0
+                while True:
+                    weights = attrs.get('WEIGHTS_' + str(i))
+                    if weights is None:
+                        break
+                    self.weights.append(gltf.accessors[weights])
+                    i += 1
+        else:
+            self.positions = positions
+            self.normals = normals
+            self.tangents = tangents
+            self.texcoords = texcoords if texcoords is not None else []
+            self.colors = colors if colors is not None else []
+            self.joints = joints if joints is not None else []
+            self.weights = weights if weights is not None else []
+
+    def render(self, gltf):
+        attributes = {}
+        if self.positions and self.positions.count:
+            attributes['POSITION'] = gltf.index('accessors', self.positions)
+        if self.normals and self.normals.count:
+            attributes['NORMAL'] = gltf.index('accessors', self.normals)
+        if self.tangents and self.tangents.count:
+            attributes['TANGENT'] = gltf.index('accessors', self.tangents)
+        for i, texcoords in enumerate(self.texcoords):
+            attributes['TEXCOORD_' + str(i)] = gltf.index('accessors', texcoords)
+        for i, colors in enumerate(self.colors):
+            attributes['COLOR_' + str(i)] = gltf.index('accessors', colors)
+        for i, joints in enumerate(self.joints):
+            attributes['JOINTS_' + str(i)] = gltf.index('accessors', joints)
+        for i, weights in enumerate(self.weights):
+            attributes['WEIGHTS_' + str(i)] = gltf.index('accessors', weights)
+        return attributes
+
+    def sort_joints(self):
+        if len(self.joints) > 1:
+            joint_set_count = len(self.joints)
+            joint_data = np.hstack(tuple(j.data for j in self.joints))
+            weight_data = np.hstack(tuple(w.data for w in self.weights))
+            sorted_weights = []
+            sorted_joints = []
+            for w, j in zip(weight_data, joint_data):
+                pairs = sorted(zip(w, j), reverse=True)
+                new_weights, new_joints = list(zip(*pairs))
+                sorted_weights.append(new_weights)
+                sorted_joints.append(new_joints)
+            sorted_weights = np.array(sorted_weights, 'float32')
+            sorted_joints = np.array(sorted_joints, 'uint16')
+            self.weights = []
+            self.joints = []
+            for i in range(joint_set_count):
+                self.weights.append(
+                    Vec4Array(sorted_weights[:, i * 4:(i + 1) * 4].astype('float32'))
+                )
+                self.joints.append(
+                    Vec4Array(sorted_joints[:, i * 4:(i + 1) * 4].astype('uint16'))
+                )
+
+    def __contains__(self, item):
+        return (self.positions == item or
+                self.normals == item or
+                self.tangents == item or
+                item in self.texcoords or
+                item in self.colors or
+                item in self.joints or
+                item in self.weights)
+
     @property
     def positions(self):
         return self._positions
@@ -307,27 +416,20 @@ class Primitive(BaseGLTFStructure):
             return
         self._tangents = tangents if isinstance(tangents, Vec4Array) else Vec4Array(tangents)
 
-    @property
-    def indices(self):
-        return self._indices
-
-    @indices.setter
-    def indices(self, indices):
-        if indices is None:
-            self._indices = None
-            return
-        self._indices = indices if isinstance(indices, ScalarArray) else ScalarArray(indices)
-
 
 class Mesh(BaseGLTFStructure):
     def __init__(self, mesh=None, gltf=None, name=None):
         self.primitives = []
         if mesh:
+            self.extras = mesh.get('extras')
             self.name = mesh.get('name')
             for prim in mesh['primitives']:
                 self.primitives.append(Primitive(prim, gltf))
+            self.weights = mesh.get('weights')
         else:
             self.name = name
+            self.extras = None
+            self.weights = None
 
     def __repr__(self):
         return self.name or super(Mesh, self).__repr__()
@@ -336,7 +438,9 @@ class Mesh(BaseGLTFStructure):
         # Note that we don't consider `name` when merging nodes
         if not isinstance(other, Mesh):
             return False
-        return self.primitives == other.primitives
+        return self.primitives == other.primitives \
+            and self.extras == other.extras \
+            and self.weights == other.weights
 
     def add_primitive(self, **kwargs):
         self.primitives.append(Primitive(**kwargs))
@@ -344,10 +448,14 @@ class Mesh(BaseGLTFStructure):
 
     def render(self, gltf):
         mesh = {
-            'primitives': [primitive.render(gltf) for primitive in self.primitives]
+            'primitives': [primitive.render(gltf) for primitive in self.primitives],
         }
         if self.name:
             mesh['name'] = self.name
+        if self.extras:
+            mesh['extras'] = self.extras
+        if self.weights:
+            mesh['weights'] = self.weights
 
         return mesh
 
@@ -402,9 +510,10 @@ class Node(BaseGLTFStructure):
     camera = None
     extensions = None
     extras = None
+    weights = None
 
     def __init__(self, node=None, gltf=None, name=None, mesh=None, children=None,
-                 skin=None, camera=None):
+                 skin=None, camera=None, weights=None):
         if node:
             self.children = []
             self.name = node.get('name')
@@ -420,6 +529,9 @@ class Node(BaseGLTFStructure):
 
             if 'extras' in node:
                 self.extras = node['extras']
+            
+            if 'weights' in node:
+                self.weights= node['weights']
 
             translation = node.get('translation')
             if translation:
@@ -445,14 +557,15 @@ class Node(BaseGLTFStructure):
             self.children = children or []
             self.skin = skin
             self.camera = camera
+            self.weights = weights or []
 
     def __repr__(self):
         return self.name or super(Node, self).__repr__()
 
     def __eq__(self, other):
         if not (isinstance(other, type(self))
-                and (self.mesh, self.name, self.children, self.skin, self.camera) ==
-                (other.mesh, other.name, other.children, other.skin, other.camera)):
+                and (self.mesh, self.name, self.children, self.skin, self.camera, self.weights) ==
+                (other.mesh, other.name, other.children, other.skin, other.camera, other.weights)):
             return False
 
         # quaternions seem to blow up if they're compared to None
@@ -561,4 +674,6 @@ class Node(BaseGLTFStructure):
             node['extensions'] = self.extensions
         if self.extras:
             node['extras'] = self.extras
+        if self.weights:
+            node['weights'] = self.weights
         return node
