@@ -291,6 +291,7 @@ def analyze(model):
         "animation_translation": 0,
         "animation_rotation": 0,
         "animation_scale": 0,
+        "animation_weights": 0,
         "textures": 0,
     }
 
@@ -866,10 +867,21 @@ def get_texture_efficiency(model):
                     tri = []
 
     efficiencies = []
+    total_pixels = 0
+    total_pixels_used = 0
     for i, t in enumerate(model.textures):
         im = ims[t.source]
-        utilization = sum(im.point(bool).getdata()) / (im.size[0] * im.size[1])
+        pixels = im.size[0] * im.size[1]
+        total_pixels += pixels
+        used = sum(im.point(bool).getdata())
+        total_pixels_used += used
+        utilization = used / pixels
         efficiencies.append(utilization)
         print(f'Texture {i}: {utilization:.0%}')
 
-    return efficiencies
+    return {
+        'efficiencies': efficiencies,
+        'total_pixels': total_pixels,
+        'total_pixels_used': total_pixels_used,
+        'overall_efficiency': total_pixels_used / total_pixels,
+    }
